@@ -1,15 +1,17 @@
-from sdkite.http.adapter import HTTPAdapter, HTTPAdapterSpec
+from sdkite.http.adapter import HTTPAdapter, HTTPAdapterSendRequest, HTTPAdapterSpec
 from sdkite.http.auth import BasicAuth, NoAuth
 from sdkite.http.model import (
     HTTPBodyEncoding,
     HTTPHeaderDict,
     HTTPRequest,
+    HTTPRequestAttemptInfo,
     HTTPResponse,
 )
 
 __all__ = (
     # sdkite.http.adapter
     "HTTPAdapter",
+    "HTTPAdapterSendRequest",
     "HTTPAdapterSpec",
     # sdkite.http.auth
     "BasicAuth",
@@ -18,5 +20,6 @@ __all__ = (
     "HTTPBodyEncoding",
     "HTTPHeaderDict",
     "HTTPRequest",
+    "HTTPRequestAttemptInfo",
     "HTTPResponse",
 )
