@@ -1,14 +1,12 @@
 from io import SEEK_END, StringIO
-from typing import TYPE_CHECKING, Any
+import sys
 
 from sdkite import AdapterSpec, Client
 
-if TYPE_CHECKING:
+if sys.version_info < (3, 11):  # pragma: no cover
     from typing_extensions import assert_type
-else:
-    # no need to have typing_extensions installed
-    def assert_type(val: Any, _: Any) -> Any:
-        return val
+else:  # pragma: no cover
+    from typing import assert_type
 
 
 class ValueSpec(AdapterSpec[StringIO]):
@@ -62,8 +60,8 @@ def test_typing() -> None:
 
 def test_complete() -> None:
     client = RootClient()
-    assert client.read.normal() == ""
-    assert client.read.upper() == ""
+    assert not client.read.normal()
+    assert not client.read.upper()
 
     client.write.append("Hello")
     assert client.read.normal() == "Hello"
@@ -74,8 +72,8 @@ def test_complete() -> None:
     assert client.read.upper() == "HELLO, WORLD!"
 
     client.write.erase()
-    assert client.read.normal() == ""
-    assert client.read.upper() == ""
+    assert not client.read.normal()
+    assert not client.read.upper()
 
 
 def test_two_clients() -> None:
@@ -95,6 +93,6 @@ class RootClientMissingValue(Client):
 
 def test_root_client_missing_value() -> None:
     client = RootClientMissingValue()
-    assert client.read.normal() == ""
+    assert not client.read.normal()
     client.write.append("Hello")
-    assert client.read.normal() == ""  # did not change
+    assert not client.read.normal()  # did not change
